if select(3, UnitClass("player")) ~= 9 then return end -- Only load when playing a warlock
local Whipped = _G.Whipped
local tinsert, sformat = table.insert, string.format
local EffectiveScale = UIParent:GetEffectiveScale()
local COMBATLOG_OBJECT_AFFILIATION_MINE = COMBATLOG_OBJECT_AFFILIATION_MINE
local Demons = Whipped.Demons
local Darkglare = Demons.Darkglare

-- Frames
local Frame_Darkglare = CreateFrame("StatusBar", nil, Whipped.MasterFrame)
local Frame_Darkglare_Icon = CreateFrame("Frame", nil, Frame_Darkglare)

function Whipped:Darkglare_SetupOptions()
	local Darkglare = {
		name = 'Darkglare',
		type = 'group',
		args = {
			Thickness = {
				name = 'Thickness',
				order = 1,
				type = 'range',
				min = 2,
				max = 200,
				softMin = 2,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.Darkglare.Thickness = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.Darkglare.Thickness end
			},
			Padding = {
				name = 'Padding',
				order = 2,
				type = 'range',
				min = 0,
				max = 200,
				softMin = 0,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.Darkglare.Padding = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.Darkglare.Padding end
			},
			Show = {
				name = 'Show',
				type = 'toggle',
				order = 4,
				set =
					function(info, input)
						self.db.global.Darkglare.Show = input;
						if input == true then
							if Whipped.Demons.Darkglare.Talented then
								Frame_Darkglare:Show()
							else
								Frame_Darkglare:Hide()
							end
						else
							Frame_Darkglare:Hide()
						end
						Whipped:UpdateAllFrames()
					end,
				get =
					function(info)
						return self.db.global.Darkglare.Show
					end
			}
		}
	}
	self:AddCategory("Darkglare", Darkglare)
end

function Whipped:Darkglare_OnLoad()
	Frame_Darkglare:SetStatusBarTexture("Interface\\TARGETINGFRAME\\UI-StatusBar")
	Frame_Darkglare:SetMinMaxValues(0, 12)
	Frame_Darkglare:SetStatusBarColor(.1,.9,.2)

	Frame_Darkglare_Icon:SetWidth(Whipped.db.global.Darkglare.Thickness * EffectiveScale)
	Frame_Darkglare_Icon:SetHeight(Whipped.db.global.Darkglare.Thickness * EffectiveScale)
	Frame_Darkglare_Icon:SetPoint(Whipped.anchors[Whipped.db.global.Anchor], Frame_Darkglare, Whipped:OppositeAnchor(Whipped.db.global.Anchor))
	
	Frame_Darkglare_Icon.texture = Frame_Darkglare_Icon:CreateTexture(nil, "BACKGROUND")
	Frame_Darkglare_Icon.texture:SetTexture("Interface\\Icons\\achievement_boss_durumu")
	Frame_Darkglare_Icon.texture:SetAllPoints(true)
	
	if not self.db.global.Darkglare.Show then Frame_Darkglare:Hide() end
end

function Whipped:Darkglare_OnUpdate()
	if Darkglare.Summoned then -- Are Summoned or on Cooldown
		local t = GetTime()
		if Darkglare.TTL > 0 then -- Living Darkglare
			Darkglare.TTL = Darkglare.CD - t - 12
			Frame_Darkglare:SetValue(Darkglare.TTL)
		else -- Dead Darkglare
			if Darkglare.TTL < 0 then -- Statement block only runs once every summon, Setup for cooldown
				Darkglare.TTL = 0
				Darkglare.Alive = 0
				--DEBUG
				-- Whipped:Print(ChatFrame3, sformat("%-15s".."|".."%-15s".."|".."%-20s".."| "..Demons.ActiveDemons(),
					-- "UNIT_DESPAWNED",
					-- "'Darkglare'",
					-- "'Summon Darkglare'"))
				------
				Frame_Darkglare:SetStatusBarColor(.1,.1,.9)
			end
			local val = (12 - (Darkglare.CD - t)) -- (CD(24) - TTL(12)) / TTL(12) = 1
			Frame_Darkglare:SetValue(val)
			if val > 12 then -- Statement block only runs once every cast, Check if off cooldown (reset the whole thing)
				Darkglare.Dead = 0
				Darkglare.GUID = 0
				Darkglare.Summoned = false
				Frame_Darkglare:SetStatusBarColor(.1,.9,.2)
			end
		end
	end
end

function Whipped:Darkglare_UpdateFrames()
	local MasterFrame = self.db.global
	local anchorID = MasterFrame.Anchor
	local anchor, Oanchor = Whipped.anchors[anchorID], Whipped:OppositeAnchor(anchorID)
	local SoulShards = MasterFrame.SoulShards
	local Imps = MasterFrame.Imps
	local Dreadstalkers = MasterFrame.Dreadstalkers
	local Cooldown = MasterFrame.Cooldown
	local Grimoire = MasterFrame.Grimoire
	local Darkglare = MasterFrame.Darkglare
	local anchorFrame, anchorHelp, x, y, temp = Whipped.Frame_Grimoire
	
	temp = Grimoire
	if not Grimoire.Show or not Demons.GrimoireDemon.Talented then
		if not Cooldown.Show then
			if not Dreadstalkers.Show then
				if not Imps.Show then
					if SoulShards.Show then
						anchorFrame = Whipped.Frame_SoulShard_1
						temp = SoulShards
					else
						anchorFrame = Whipped.MasterFrame
						temp = MasterFrame
					end
				else
					anchorFrame = Whipped.Frame_Imps_1
					temp = Imps
				end
			else
				anchorFrame = Whipped.Frame_Dreadstalkers
				temp = Dreadstalkers
			end
		else
			anchorFrame = Whipped.Frame_Cooldown
			temp = Cooldown
		end
	end
	
	if anchorID == 1 or anchorID == 2 then
		Frame_Darkglare:SetWidth((SoulShards.Width * 5 + SoulShards.Padding * 4) * EffectiveScale)
		Frame_Darkglare:SetHeight(Darkglare.Thickness * EffectiveScale)
		if anchorID == 1 then
			if temp == MasterFrame then
				anchorHelp, x, y = "TOP", MasterFrame.PaddingX * EffectiveScale, -(MasterFrame.PaddingY + Darkglare.Padding) * EffectiveScale
			else
				anchorHelp, x, y = "BOTTOM", 0, -(temp.Padding + Darkglare.Padding) * EffectiveScale
			end
		else
			if temp == MasterFrame then
				anchorHelp, x, y = "TOP", -MasterFrame.PaddingX * EffectiveScale, -(MasterFrame.PaddingY + Darkglare.Padding) * EffectiveScale
			else
				anchorHelp, x, y = "BOTTOM", 0, -(temp.Padding + Darkglare.Padding) * EffectiveScale
			end
		end
	else
		Frame_Darkglare:SetWidth(Darkglare.Thickness * EffectiveScale)
		Frame_Darkglare:SetHeight((SoulShards.Height * 5 + SoulShards.Padding * 4) * EffectiveScale)
		if anchorID == 3 then
			if temp == MasterFrame then
				anchorHelp, x, y = "LEFT", (MasterFrame.PaddingX + Darkglare.Padding) * EffectiveScale, -MasterFrame.PaddingY * EffectiveScale
			else
				anchorHelp, x, y = "RIGHT", (temp.Padding + Darkglare.Padding) * EffectiveScale, 0
			end
		else
			if temp == MasterFrame then
				anchorHelp, x, y = "LEFT", (MasterFrame.PaddingX + Dreadstalker.Padding) * EffectiveScale, -MasterFrame.PaddingY * EffectiveScale
			else
				anchorHelp, x, y = "RIGHT", (temp.Padding + Darkglare.Padding) * EffectiveScale, 0
			end
		end
	end
	Frame_Darkglare_Icon:SetWidth(Darkglare.Thickness * EffectiveScale)
	Frame_Darkglare_Icon:SetHeight(Darkglare.Thickness * EffectiveScale)
		
	Frame_Darkglare:ClearAllPoints()
	Frame_Darkglare_Icon:ClearAllPoints()
	if anchorID == 1 then
		Frame_Darkglare:SetPoint("TOP"..anchor, anchorFrame, anchorHelp..anchor, x, y)
		Frame_Darkglare_Icon:SetPoint(anchor, Frame_Darkglare, Oanchor, MasterFrame.PaddingX * EffectiveScale, 0)
		Frame_Darkglare:SetOrientation("HORIZONTAL")
	elseif anchorID == 2 then
		Frame_Darkglare:SetPoint("TOP"..anchor, anchorFrame, anchorHelp..anchor, x, y)
		Frame_Darkglare_Icon:SetPoint(anchor, Frame_Darkglare, Oanchor, -MasterFrame.PaddingX * EffectiveScale, 0)
		Frame_Darkglare:SetOrientation("HORIZONTAL")
	elseif anchorID == 3 then
		Frame_Darkglare:SetPoint(anchor.."LEFT", anchorFrame, anchor..anchorHelp, x, y)
		Frame_Darkglare_Icon:SetPoint(anchor, Frame_Darkglare, Oanchor, 0, -MasterFrame.PaddingY * EffectiveScale)
		Frame_Darkglare:SetOrientation("VERTICAL")
	elseif anchorID ==4 then
		Frame_Darkglare:SetPoint(anchor.."LEFT", anchorFrame, anchor..anchorHelp, x, y)
		Frame_Darkglare_Icon:SetPoint(anchor, Frame_Darkglare, Oanchor, 0, MasterFrame.PaddingY * EffectiveScale)
		Frame_Darkglare:SetOrientation("VERTICAL")
	end
end

function Whipped:Darkglare_CLE(...)
	local event = select(2, ...)
	if event == "SPELL_CAST_SUCCESS" then
		if Darkglare.Alive > 0 then
			Frame_Darkglare:SetStatusBarColor(1,.1,.1)
		end
	elseif event == "SPELL_SUMMON" then
		local SpellID = select(12, ...)
		if SpellID == 205180 and bit.band(select(6,...), COMBATLOG_OBJECT_AFFILIATION_MINE) > 0 then -- Darkglare
			Darkglare.Summoned = true
			Darkglare.CD = GetTime() + 24
			Darkglare.TTL = Darkglare.CD - 12
			Darkglare.Alive = 1
			Darkglare.DE = 1
			Darkglare.GUID = select(8, ...)
			--DEBUG
			-- Whipped:Print(ChatFrame3, sformat("%-15s".."|".."%-15s".."|".."%-20s".."| "..Demons.ActiveDemons(),
				-- event,
				-- "'Darkglare'",
				-- "'Summon Darkglare'"))
			------
		end
	elseif event == "UNIT_DIED" then
		if select(8,...) == Demons.Darkglare.GUID then -- Darkglare
			Darkglare.Alive = 0
			Darkglare.TTL = 0
			Frame_Darkglare:SetStatusBarColor(.1,.1,.9)
			--DEBUG
			-- Whipped:Print(ChatFrame3, sformat("%-15s".."|".."%-15s".."|".."%-20s".."| "..Demons.ActiveDemons(),
				-- event,
				-- "'Darkglare'",
				-- "'Summon Darkglare'"))
			------
		end
	end
end

function Whipped:Darkglare_UpdateTalents()
	local id,_,_,b = GetTalentInfo(7,1,1)
	if id == 22478 and b then
		Darkglare.Talented = true
		if self.db.global.Darkglare.Show then
			Frame_Darkglare:Show()
		end
	else
		Darkglare.Talented = false
		Frame_Darkglare:Hide()
	end
	--DEBUG
	-- Whipped:Print(ChatFrame3, sformat("%-15s".."|".."%-36s".."| "..tostring(Demons.Darkglare.Talented),
		-- "TALENT_UPDATE",
		-- "'Darkglare'"))
	------
end