if select(3, UnitClass("player")) ~= 9 then return end -- Only load when playing a warlock
local Whipped = _G.Whipped
local EffectiveScale = UIParent:GetEffectiveScale()
local sformat = string.format -- DEBUG
local COMBATLOG_OBJECT_AFFILIATION_MINE = COMBATLOG_OBJECT_AFFILIATION_MINE
local Demons = Whipped.Demons
local Dreadstalkers = Demons.Dreadstalkers

-- Frames
Whipped.Frame_Dreadstalkers = CreateFrame("StatusBar", nil, Whipped.MasterFrame)
local Frame_Dreadstalkers = Whipped.Frame_Dreadstalkers
local Frame_Dreadstalkers_Icon = CreateFrame("Frame", nil, Frame_Dreadstalkers)

function Whipped:Dreadstalkers_SetupOptions()
	local Dreadstalkers = {
		name = 'Dreadstalkers',
		type = 'group',
		args = {
			Thickness = {
				name = 'Thickness',
				order = 1,
				type = 'range',
				min = 2,
				max = 200,
				softMin = 2,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.Dreadstalkers.Thickness = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.Dreadstalkers.Thickness end
			},
			Padding = {
				name = 'Padding',
				order = 3,
				type = 'range',
				min = 0,
				max = 200,
				softMin = 0,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.Dreadstalkers.Padding = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.Dreadstalkers.Padding end
			},
			Show = {
				name = 'Show',
				type = 'toggle',
				order = 4,
				set =
					function(info, input)
						self.db.global.Dreadstalkers.Show = input;
						if input == true then
							Frame_Dreadstalkers:Show()
						else
							Frame_Dreadstalkers:Hide()
						end
						Whipped:UpdateAllFrames()
					end,
				get =
					function(info)
						return self.db.global.Dreadstalkers.Show
					end
			}
		}
	}
	self:AddCategory("Dreadstalkers", Dreadstalkers)
end

function Whipped:Dreadstalkers_OnLoad()
	Frame_Dreadstalkers:SetStatusBarTexture("Interface\\TARGETINGFRAME\\UI-StatusBar")
	Frame_Dreadstalkers:SetMinMaxValues(0, 12)
	Frame_Dreadstalkers:SetStatusBarColor(.1,.9,.2)

	Frame_Dreadstalkers_Icon:SetWidth(Whipped.db.global.Dreadstalkers.Thickness * EffectiveScale)
	Frame_Dreadstalkers_Icon:SetHeight(Whipped.db.global.Dreadstalkers.Thickness * EffectiveScale)
	Frame_Dreadstalkers_Icon:SetPoint(Whipped.anchors[Whipped.db.global.Anchor], Frame_Dreadstalkers, Whipped:OppositeAnchor(Whipped.db.global.Anchor))
	
	Frame_Dreadstalkers_Icon.texture = Frame_Dreadstalkers_Icon:CreateTexture(nil, "BACKGROUND")
	Frame_Dreadstalkers_Icon.texture:SetTexture("Interface\\Icons\\Spell_warlock_calldreadstalkers")
	Frame_Dreadstalkers_Icon.texture:SetAllPoints(true)
	
	if not self.db.global.Dreadstalkers.Show then Frame_Dreadstalkers:Hide() end
end

function Whipped:Dreadstalkers_OnUpdate()
	if Dreadstalkers.Summoned then -- Are Summoned or on Cooldown
		local t = GetTime()
		if Dreadstalkers.TTL > 0 then -- Living Dreadstalkers
			Dreadstalkers.TTL = Dreadstalkers.CD - t - 3
			Frame_Dreadstalkers:SetValue(Dreadstalkers.TTL)
		else -- Dead Dreadstalkers
			if Dreadstalkers.TTL < 0 then -- Statement block only runs once every summon, Setup for cooldown
				Dreadstalkers.TTL = 0
				Dreadstalkers.Alive = 0
				--DEBUG
				-- if Dreadstalkers.Dead < 2 then
					-- Whipped:Print(ChatFrame3, sformat("%-15s".."|".."%-15s".."|".."%-20s".."| "..Demons.ActiveDemons(),
						-- "UNIT_DESPAWNED",
						-- "'Dreadstalkers'",
						-- "'Call Dreadstalkers'"))
				-- end
				------
				Frame_Dreadstalkers:SetStatusBarColor(.1,.1,.9)
				Whipped:ImprovedDreadstalkers()
			end
			local val = (3 - (Dreadstalkers.CD - t)) / .25 -- (CD(15) - TTL(12)) / TTL(12) = .25
			Frame_Dreadstalkers:SetValue(val)
			if val > 12 then -- Statement block only runs once every cast, Check if off cooldown (reset the whole thing)
				Dreadstalkers.Dead = 0
				Dreadstalkers.GUID1 = 0
				Dreadstalkers.Summoned = false
				Frame_Dreadstalkers:SetStatusBarColor(.1,.9,.2)
			end
		end
	end
end

function Whipped:Dreadstalkers_UpdateFrames()
	local MasterFrame = self.db.global
	local anchorID = MasterFrame.Anchor
	local anchor, Oanchor = Whipped.anchors[anchorID], Whipped:OppositeAnchor(anchorID)
	local SoulShards = MasterFrame.SoulShards
	local Imps = MasterFrame.Imps
	local Dreadstalkers = MasterFrame.Dreadstalkers
	local anchorFrame, anchorHelp, x, y, temp = Whipped.Frame_Imps_1
	
	temp = Imps
	if not Imps.Show then
		if SoulShards.Show then
			anchorFrame = Whipped.Frame_SoulShard_1
			temp = SoulShards
		else
			anchorFrame = Whipped.MasterFrame
			temp = MasterFrame
		end
	end
	
	if anchorID == 1 or anchorID == 2 then
		Frame_Dreadstalkers:SetWidth((SoulShards.Width * 5 + SoulShards.Padding * 4) * EffectiveScale)
		Frame_Dreadstalkers:SetHeight(Dreadstalkers.Thickness * EffectiveScale)
		if anchorID == 1 then
			if temp == MasterFrame then
				anchorHelp, x, y = "TOP", MasterFrame.PaddingX * EffectiveScale, -(MasterFrame.PaddingY + Dreadstalkers.Padding) * EffectiveScale
			else
				anchorHelp, x, y = "BOTTOM", 0, -(temp.Padding + Dreadstalkers.Padding) * EffectiveScale
			end
		else
			if temp == MasterFrame then
				anchorHelp, x, y = "TOP", -MasterFrame.PaddingX * EffectiveScale, -(MasterFrame.PaddingY + Dreadstalkers.Padding) * EffectiveScale
			else
				anchorHelp, x, y = "BOTTOM", 0, -(temp.Padding + Dreadstalkers.Padding) * EffectiveScale
			end
		end
	else
		Frame_Dreadstalkers:SetWidth(Dreadstalkers.Thickness * EffectiveScale)
		Frame_Dreadstalkers:SetHeight((SoulShards.Height * 5 + SoulShards.Padding * 4) * EffectiveScale)
		if anchorID == 3 then
			if temp == MasterFrame then
				anchorHelp, x, y = "LEFT", (MasterFrame.PaddingX + Dreadstalkers.Padding) * EffectiveScale, -MasterFrame.PaddingY * EffectiveScale
			else
				anchorHelp, x, y = "RIGHT", (temp.Padding + Dreadstalkers.Padding) * EffectiveScale, 0
			end
		else
			if temp == MasterFrame then
				anchorHelp, x, y = "LEFT", (MasterFrame.PaddingX + Dreadstalkers.Padding) * EffectiveScale, -MasterFrame.PaddingY * EffectiveScale
			else
				anchorHelp, x, y = "RIGHT", (temp.Padding + Dreadstalkers.Padding) * EffectiveScale, 0
			end
		end
	end
	Frame_Dreadstalkers_Icon:SetWidth(Dreadstalkers.Thickness * EffectiveScale)
	Frame_Dreadstalkers_Icon:SetHeight(Dreadstalkers.Thickness * EffectiveScale)
		
	Frame_Dreadstalkers:ClearAllPoints()
	Frame_Dreadstalkers_Icon:ClearAllPoints()
	if anchorID == 1 then
		Frame_Dreadstalkers:SetPoint("TOP"..anchor, anchorFrame, anchorHelp..anchor, x, y)
		Frame_Dreadstalkers_Icon:SetPoint(anchor, Frame_Dreadstalkers, Oanchor, MasterFrame.PaddingX * EffectiveScale, 0)
		Frame_Dreadstalkers:SetOrientation("HORIZONTAL")
	elseif anchorID == 2 then
		Frame_Dreadstalkers:SetPoint("TOP"..anchor, anchorFrame, anchorHelp..anchor, x, y)
		Frame_Dreadstalkers_Icon:SetPoint(anchor, Frame_Dreadstalkers, Oanchor, -MasterFrame.PaddingX * EffectiveScale, 0)
		Frame_Dreadstalkers:SetOrientation("HORIZONTAL")
	elseif anchorID == 3 then
		Frame_Dreadstalkers:SetPoint(anchor.."LEFT", anchorFrame, anchor..anchorHelp, x, y)
		Frame_Dreadstalkers_Icon:SetPoint(anchor, Frame_Dreadstalkers, Oanchor, 0, -MasterFrame.PaddingY * EffectiveScale)
		Frame_Dreadstalkers:SetOrientation("VERTICAL")
	elseif anchorID ==4 then
		Frame_Dreadstalkers:SetPoint(anchor.."LEFT", anchorFrame, anchor..anchorHelp, x, y)
		Frame_Dreadstalkers_Icon:SetPoint(anchor, Frame_Dreadstalkers, Oanchor, 0, MasterFrame.PaddingY * EffectiveScale)
		Frame_Dreadstalkers:SetOrientation("VERTICAL")
	end
end

function Whipped:Dreadstalkers_CLE(...)
	local timestamp, event, playerGUID = select(1, ...), select(2, ...), select(4, ...)
	local SpellName
	if event == "SPELL_CAST_SUCCESS" then
		if Dreadstalkers.Alive > 0 then
			Frame_Dreadstalkers:SetStatusBarColor(1,.1,.1)
		end
	elseif event == "SPELL_SUMMON" then
		if (select(12, ...) == 193331 or select(12, ...) == 193332) and bit.band(select(6,...), COMBATLOG_OBJECT_AFFILIATION_MINE) > 0 then
			if not Dreadstalkers.Summoned then
				Dreadstalkers.Summoned = true
				Dreadstalkers.CD = GetTime() + 15
				Dreadstalkers.TTL = Dreadstalkers.CD - 3
				Dreadstalkers.Alive = 2
				Dreadstalkers.DE = 1
				--DEBUG
				-- Whipped:Print(ChatFrame3, sformat("%-15s".."|".."%-15s".."|".."%-20s".."| "..Demons.ActiveDemons(),
					-- event,
					-- "'Dreadstalkers'",
					-- "'Call Dreadstalkers'"))
				--
			end
			if Dreadstalkers.GUID1 == 0 then -- Saves Dreadstalker GUIDs to track UNIT_DIED events (UNIT_DIED has no owner data for pets)
				Dreadstalkers.GUID1 = select(8,...)
			else
				Dreadstalkers.GUID2 = select(8,...)
			end
		end
	elseif event == "UNIT_DIED" then
		if select(8,...) == Dreadstalkers.GUID1 or select(8,...) == Dreadstalkers.GUID2 then
			Dreadstalkers.Dead = Dreadstalkers.Dead + 1
			Dreadstalkers.Alive = Dreadstalkers.Alive - 1
			if Dreadstalkers.Alive == 0 then
				Frame_Dreadstalkers:SetStatusBarColor(.1,.1,.9)
			end
			--DEBUG
			-- Whipped:Print(ChatFrame3, sformat("%-15s".."|".."%-15s".."|".."%-20s".."| "..Demons.ActiveDemons(),
				-- event,
				-- "'Dreadstalkers'",
				-- "'Call Dreadstalkers'"))
			------
		end
	elseif event == "SPELL_AURA_APPLIED" then
		if bit.band(select(6,...), COMBATLOG_OBJECT_AFFILIATION_MINE) > 0 then
			if select(12,...) ==205146 then -- Demonic Calling
				ActionButton_ShowOverlayGlow(Frame_Dreadstalkers_Icon)
			end
		end
	elseif event == "SPELL_AURA_REMOVED" then
		if bit.band(select(6,...), COMBATLOG_OBJECT_AFFILIATION_MINE) > 0 then
			if select(12,...) == 205146 then -- Demonic Calling
				ActionButton_HideOverlayGlow(Frame_Dreadstalkers_Icon)
			end
		end
	end
end