if select(3, UnitClass("player")) ~= 9 then return end -- Only load when playing a warlock
local Whipped = _G.Whipped
local tinsert, sformat = table.insert, string.format
local EffectiveScale = UIParent:GetEffectiveScale()
local COMBATLOG_OBJECT_AFFILIATION_MINE = COMBATLOG_OBJECT_AFFILIATION_MINE
local Demons = Whipped.Demons
local GrimoireDemon = Demons.GrimoireDemon

-- Frames
Whipped.Frame_Grimoire = CreateFrame("StatusBar", nil, Whipped.MasterFrame)
local Frame_Grimoire = Whipped.Frame_Grimoire
local Frame_Grimoire_Icon = CreateFrame("Frame", nil, Frame_Grimoire)

function Whipped:Grimoire_SetupOptions()
	local Grimoire = {
		name = 'Grimoire',
		type = 'group',
		args = {
			Thickness = {
				name = 'Thickness',
				order = 1,
				type = 'range',
				min = 2,
				max = 200,
				softMin = 2,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.Grimoire.Thickness = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.Grimoire.Thickness end
			},
			Padding = {
				name = 'Padding',
				order = 3,
				type = 'range',
				min = 0,
				max = 200,
				softMin = 0,
				softMax = 100,
				step = .1,
				set = function(info, input) self.db.global.Grimoire.Padding = input; Whipped:UpdateAllFrames() end,
				get = function(info) return self.db.global.Grimoire.Padding end
			},
			Show = {
				name = 'Show',
				type = 'toggle',
				order = 4,
				set =
					function(info, input)
						self.db.global.Grimoire.Show = input;
						if input == true then
							if Whipped.Demons.GrimoireDemon.Talented then
								Frame_Grimoire:Show()
							else
								Frame_Grimoire:Hide()
							end
						else
							Frame_Grimoire:Hide()
						end
						Whipped:UpdateAllFrames()
					end,
				get =
					function(info)
						return self.db.global.Grimoire.Show
					end
			}
		}
	}
	self:AddCategory("Grimoire", Grimoire)
end

function Whipped:Grimoire_OnLoad()
	Frame_Grimoire:SetStatusBarTexture("Interface\\TARGETINGFRAME\\UI-StatusBar")
	Frame_Grimoire:SetMinMaxValues(0, 25)
	Frame_Grimoire:SetStatusBarColor(.1,.9,.2)
	
	--Frame_Grimoire.bg = Frame_Grimoire:CreateTexture(nil, "BACKGROUND")
	--Frame_Grimoire.bg:SetTexture("Interface\\TARGETINGFRAME\\UI-StatusBar")
	--Frame_Grimoire.bg:SetVertexColor(.1,.9,.9)
	
	Frame_Grimoire_Icon:SetWidth(Whipped.db.global.Grimoire.Thickness * EffectiveScale)
	Frame_Grimoire_Icon:SetHeight(Whipped.db.global.Grimoire.Thickness * EffectiveScale)
	Frame_Grimoire_Icon:SetPoint(Whipped.anchors[Whipped.db.global.Anchor], Frame_Grimoire, Whipped:OppositeAnchor(Whipped.db.global.Anchor))
	
	Frame_Grimoire_Icon.texture = Frame_Grimoire_Icon:CreateTexture(nil, "BACKGROUND")
	Frame_Grimoire_Icon.texture:SetTexture("Interface\\Icons\\warlock_grimoireofcommand")
	Frame_Grimoire_Icon.texture:SetAllPoints(true)
	
	if not self.db.global.Grimoire.Show then Frame_Grimoire:Hide() end
end

function Whipped:Grimoire_OnUpdate()
	if GrimoireDemon.Summoned then -- Are Summoned or on Cooldown
		local t = GetTime()
		if GrimoireDemon.TTL > 0 then -- Living GrimoireDemon
			GrimoireDemon.TTL = GrimoireDemon.CD - t - 65
			Frame_Grimoire:SetValue(GrimoireDemon.TTL)
		else -- Dead GrimoireDemon
			if GrimoireDemon.TTL < 0 then -- Statement block only runs once every summon, Setup for cooldown
				GrimoireDemon.TTL = 0
				GrimoireDemon.Alive = 0
				--DEBUG
				-- Whipped:Print(ChatFrame3, sformat("%-15s".."|".."%-15s".."|".."%-20s".."| "..Demons.ActiveDemons(),
					-- "UNIT_DESPAWNED",
					-- "'"..GrimoireDemon.Name.."'",
					-- "'Grimoire: "..GrimoireDemon.Name.."'"))
				------
				Frame_Grimoire:SetStatusBarColor(.1,.1,.9)
			end
			local val = (65 - (GrimoireDemon.CD - t)) / 2.6 -- (CD(90) - TTL(25)) / TTL(25) = 2.6
			Frame_Grimoire:SetValue(val)
			if val > 25 then -- Statement block only runs once every cast, Check if off cooldown (reset the whole thing)
				GrimoireDemon.Dead = 0
				GrimoireDemon.GUID = 0
				GrimoireDemon.Summoned = false
				Frame_Grimoire:SetStatusBarColor(.1,.9,.2)
			end
		end
	end
end

function Whipped:Grimoire_UpdateFrames()
	local MasterFrame = self.db.global
	local anchorID = MasterFrame.Anchor
	local anchor, Oanchor = Whipped.anchors[anchorID], Whipped:OppositeAnchor(anchorID)
	local SoulShards = MasterFrame.SoulShards
	local Imps = MasterFrame.Imps
	local Dreadstalkers = MasterFrame.Dreadstalkers
	local Cooldown = MasterFrame.Cooldown
	local Grimoire = MasterFrame.Grimoire
	local anchorFrame, anchorHelp, x, y, temp = Whipped.Frame_Cooldown
	
	temp = Cooldown
	if not Cooldown.Show then
		if not Dreadstalkers.Show then
			if not Imps.Show then
				if SoulShards.Show then
					anchorFrame = Whipped.Frame_SoulShard_1
					temp = SoulShards
				else
					anchorFrame = Whipped.MasterFrame
					temp = MasterFrame
				end
			else
				anchorFrame = Whipped.Frame_Imps_1
				temp = Imps
			end
		else
			anchorFrame = Whipped.Frame_Dreadstalkers
			temp = Dreadstalkers
		end
	end
	
	if anchorID == 1 or anchorID == 2 then
		Frame_Grimoire:SetWidth((SoulShards.Width * 5 + SoulShards.Padding * 4) * EffectiveScale)
		Frame_Grimoire:SetHeight(Grimoire.Thickness * EffectiveScale)
		if anchorID == 1 then
			if temp == MasterFrame then
				anchorHelp, x, y = "TOP", MasterFrame.PaddingX * EffectiveScale, -(MasterFrame.PaddingY + Grimoire.Padding) * EffectiveScale
			else
				anchorHelp, x, y = "BOTTOM", 0, -(temp.Padding + Grimoire.Padding) * EffectiveScale
			end
		else
			if temp == MasterFrame then
				anchorHelp, x, y = "TOP", -MasterFrame.PaddingX * EffectiveScale, -(MasterFrame.PaddingY + Grimoire.Padding) * EffectiveScale
			else
				anchorHelp, x, y = "BOTTOM", 0, -(temp.Padding + Grimoire.Padding) * EffectiveScale
			end
		end
	else
		Frame_Grimoire:SetWidth(Grimoire.Thickness * EffectiveScale)
		Frame_Grimoire:SetHeight((SoulShards.Height * 5 + SoulShards.Padding * 4) * EffectiveScale)
		if anchorID == 3 then
			if temp == MasterFrame then
				anchorHelp, x, y = "LEFT", (MasterFrame.PaddingX + Grimoire.Padding) * EffectiveScale, -MasterFrame.PaddingY * EffectiveScale
			else
				anchorHelp, x, y = "RIGHT", (temp.Padding + Grimoire.Padding) * EffectiveScale, 0
			end
		else
			if temp == MasterFrame then
				anchorHelp, x, y = "LEFT", (MasterFrame.PaddingX + Dreadstalker.Padding) * EffectiveScale, -MasterFrame.PaddingY * EffectiveScale
			else
				anchorHelp, x, y = "RIGHT", (temp.Padding + Grimoire.Padding) * EffectiveScale, 0
			end
		end
	end
	Frame_Grimoire_Icon:SetWidth(Grimoire.Thickness * EffectiveScale)
	Frame_Grimoire_Icon:SetHeight(Grimoire.Thickness * EffectiveScale)
		
	Frame_Grimoire:ClearAllPoints()
	Frame_Grimoire_Icon:ClearAllPoints()
	if anchorID == 1 then
		Frame_Grimoire:SetPoint("TOP"..anchor, anchorFrame, anchorHelp..anchor, x, y)
		Frame_Grimoire_Icon:SetPoint(anchor, Frame_Grimoire, Oanchor, MasterFrame.PaddingX * EffectiveScale, 0)
		Frame_Grimoire:SetOrientation("HORIZONTAL")
	elseif anchorID == 2 then
		Frame_Grimoire:SetPoint("TOP"..anchor, anchorFrame, anchorHelp..anchor, x, y)
		Frame_Grimoire_Icon:SetPoint(anchor, Frame_Grimoire, Oanchor, -MasterFrame.PaddingX * EffectiveScale, 0)
		Frame_Grimoire:SetOrientation("HORIZONTAL")
	elseif anchorID == 3 then
		Frame_Grimoire:SetPoint(anchor.."LEFT", anchorFrame, anchor..anchorHelp, x, y)
		Frame_Grimoire_Icon:SetPoint(anchor, Frame_Grimoire, Oanchor, 0, -MasterFrame.PaddingY * EffectiveScale)
		Frame_Grimoire:SetOrientation("VERTICAL")
	elseif anchorID ==4 then
		Frame_Grimoire:SetPoint(anchor.."LEFT", anchorFrame, anchor..anchorHelp, x, y)
		Frame_Grimoire_Icon:SetPoint(anchor, Frame_Grimoire, Oanchor, 0, MasterFrame.PaddingY * EffectiveScale)
		Frame_Grimoire:SetOrientation("VERTICAL")
	end
end

function Whipped:Grimoire_CLE(...)
	local event = select(2, ...)
	if event == "SPELL_CAST_SUCCESS" then
		if GrimoireDemon.Alive > 0 then
			Frame_Grimoire:SetStatusBarColor(1,.1,.1)
		end
	elseif event == "SPELL_AURA_REMOVED" then
		if GrimoireDemon.TTL > 0 then
			Frame_Grimoire:SetStatusBarColor(.1,.9,.2)
		end
	elseif event == "SPELL_SUMMON" then
		local SpellID = select(12, ...)
		if (SpellID == 111859 or SpellID == 111895 or SpellID == 111896 or SpellID == 111897 or SpellID == 111898)
			and bit.band(select(6,...), COMBATLOG_OBJECT_AFFILIATION_MINE) > 0 then
		-- Grimoire: Imp, Grimoire: Voidwalker, Grimoire: Succubus, Grimoire: Felhunter, Grimoire: Felguard
			GrimoireDemon.Summoned = true
			GrimoireDemon.CD = GetTime() + 90
			GrimoireDemon.TTL = GrimoireDemon.CD - 65
			GrimoireDemon.Alive = 1
			GrimoireDemon.DE = 1
			GrimoireDemon.Name = select(9, ...)
			GrimoireDemon.GUID = select(8, ...)
			--DEBUG
			-- Whipped:Print(ChatFrame3, sformat("%-15s".."|".."%-15s".."|".."%-20s".."| "..Demons.ActiveDemons(),
				-- event,
				-- "'"..select(9,...).."'",
				-- "'"..select(13,...).."'"))
			------
		end
	elseif event == "UNIT_DIED" then
		if select(8,...) == Demons.GrimoireDemon.GUID then -- Grimoire: Felguard
			GrimoireDemon.Alive = 0
			GrimoireDemon.TTL = 0
			Frame_Grimoire:SetStatusBarColor(.1,.1,.9)
			--DEBUG
			-- Whipped:Print(ChatFrame3, sformat("%-15s".."|".."%-15s".."|".."%-20s".."| "..Demons.ActiveDemons(),
				-- event,
				-- "'"..select(9, ...).."'",
				-- "'Grimoire: Felguard'"))
			------
		end
	end
end

function Whipped:Grimoire_UpdateTalents()
	local id,_,_,b = GetTalentInfo(6,2,1)
	if id == 19294 and b then
		GrimoireDemon.Talented = true
		if self.db.global.Grimoire.Show then
			Frame_Grimoire:Show()
		end
	else
		GrimoireDemon.Talented = false
		Frame_Grimoire:Hide()
	end
	--DEBUG
	-- Whipped:Print(ChatFrame3, sformat("%-15s".."|".."%-36s".."| "..tostring(Demons.GrimoireDemon.Talented),
		-- "TALENT_UPDATE",
		-- "'Grimoire of Service'"))
	------
end